from typing import Any

class ADC:
    def __init__(self, *argv, **kwargs) -> None: ...
    def read(self, *args, **kwargs) -> Any: ...
    def read_u16(self, *args, **kwargs) -> Any: ...

DEEPSLEEP: int
DEEPSLEEP_RESET: int
HARD_RESET: int

class I2C:
    def __init__(self, *argv, **kwargs) -> None: ...
    def readinto(self, *args, **kwargs) -> Any: ...
    def start(self, *args, **kwargs) -> Any: ...
    def stop(self, *args, **kwargs) -> Any: ...
    def write(self, *args, **kwargs) -> Any: ...
    def init(self, *args, **kwargs) -> Any: ...
    def readfrom(self, *args, **kwargs) -> Any: ...
    def readfrom_into(self, *args, **kwargs) -> Any: ...
    def readfrom_mem(self, *args, **kwargs) -> Any: ...
    def readfrom_mem_into(self, *args, **kwargs) -> Any: ...
    def scan(self, *args, **kwargs) -> Any: ...
    def writeto(self, *args, **kwargs) -> Any: ...
    def writeto_mem(self, *args, **kwargs) -> Any: ...
    def writevto(self, *args, **kwargs) -> Any: ...

class PWM:
    def __init__(self, *argv, **kwargs) -> None: ...
    def deinit(self, *args, **kwargs) -> Any: ...
    def duty(self, *args, **kwargs) -> Any: ...
    def freq(self, *args, **kwargs) -> Any: ...
    def init(self, *args, **kwargs) -> Any: ...

PWRON_RESET: int

class Pin:
    def __init__(self, *argv, **kwargs) -> None: ...
    def value(self, *args, **kwargs) -> Any: ...
    IN: int
    IRQ_FALLING: int
    IRQ_RISING: int
    OPEN_DRAIN: int
    OUT: int
    PULL_UP: int
    def init(self, *args, **kwargs) -> Any: ...
    def irq(self, *args, **kwargs) -> Any: ...
    def off(self, *args, **kwargs) -> Any: ...
    def on(self, *args, **kwargs) -> Any: ...

class RTC:
    def __init__(self, *argv, **kwargs) -> None: ...
    ALARM0: int
    def alarm(self, *args, **kwargs) -> Any: ...
    def alarm_left(self, *args, **kwargs) -> Any: ...
    def datetime(self, *args, **kwargs) -> Any: ...
    def irq(self, *args, **kwargs) -> Any: ...
    def memory(self, *args, **kwargs) -> Any: ...

SOFT_RESET: int

class SPI:
    def __init__(self, *argv, **kwargs) -> None: ...
    def read(self, *args, **kwargs) -> Any: ...
    def readinto(self, *args, **kwargs) -> Any: ...
    def write(self, *args, **kwargs) -> Any: ...
    LSB: int
    MSB: int
    def deinit(self, *args, **kwargs) -> Any: ...
    def init(self, *args, **kwargs) -> Any: ...
    def write_readinto(self, *args, **kwargs) -> Any: ...

class Signal:
    def __init__(self, *argv, **kwargs) -> None: ...
    def value(self, *args, **kwargs) -> Any: ...
    def off(self, *args, **kwargs) -> Any: ...
    def on(self, *args, **kwargs) -> Any: ...

class SoftI2C:
    def __init__(self, *argv, **kwargs) -> None: ...
    def readinto(self, *args, **kwargs) -> Any: ...
    def start(self, *args, **kwargs) -> Any: ...
    def stop(self, *args, **kwargs) -> Any: ...
    def write(self, *args, **kwargs) -> Any: ...
    def init(self, *args, **kwargs) -> Any: ...
    def readfrom(self, *args, **kwargs) -> Any: ...
    def readfrom_into(self, *args, **kwargs) -> Any: ...
    def readfrom_mem(self, *args, **kwargs) -> Any: ...
    def readfrom_mem_into(self, *args, **kwargs) -> Any: ...
    def scan(self, *args, **kwargs) -> Any: ...
    def writeto(self, *args, **kwargs) -> Any: ...
    def writeto_mem(self, *args, **kwargs) -> Any: ...
    def writevto(self, *args, **kwargs) -> Any: ...

class SoftSPI:
    def __init__(self, *argv, **kwargs) -> None: ...
    def read(self, *args, **kwargs) -> Any: ...
    def readinto(self, *args, **kwargs) -> Any: ...
    def write(self, *args, **kwargs) -> Any: ...
    LSB: int
    MSB: int
    def deinit(self, *args, **kwargs) -> Any: ...
    def init(self, *args, **kwargs) -> Any: ...
    def write_readinto(self, *args, **kwargs) -> Any: ...

class Timer:
    def __init__(self, *argv, **kwargs) -> None: ...
    ONE_SHOT: int
    PERIODIC: int
    def deinit(self, *args, **kwargs) -> Any: ...
    def init(self, *args, **kwargs) -> Any: ...

class UART:
    def __init__(self, *argv, **kwargs) -> None: ...
    def any(self, *args, **kwargs) -> Any: ...
    def read(self, *args, **kwargs) -> Any: ...
    def readinto(self, *args, **kwargs) -> Any: ...
    def readline(self, *args, **kwargs) -> Any: ...
    def write(self, *args, **kwargs) -> Any: ...
    def init(self, *args, **kwargs) -> Any: ...

class WDT:
    def __init__(self, *argv, **kwargs) -> None: ...
    def deinit(self, *args, **kwargs) -> Any: ...
    def feed(self, *args, **kwargs) -> Any: ...

WDT_RESET: int

def bitstream(*args, **kwargs) -> Any: ...
def deepsleep(*args, **kwargs) -> Any: ...
def disable_irq(*args, **kwargs) -> Any: ...
def enable_irq(*args, **kwargs) -> Any: ...
def freq(*args, **kwargs) -> Any: ...
def idle(*args, **kwargs) -> Any: ...
def lightsleep(*args, **kwargs) -> Any: ...

mem16: Any
mem32: Any
mem8: Any

def reset(*args, **kwargs) -> Any: ...
def reset_cause(*args, **kwargs) -> Any: ...
def sleep(*args, **kwargs) -> Any: ...
def soft_reset(*args, **kwargs) -> Any: ...
def time_pulse_us(*args, **kwargs) -> Any: ...
def unique_id(*args, **kwargs) -> Any: ...



"""
Module: 'network' on micropython-v1.18-esp8266
"""
# MCU: {'ver': 'v1.18', 'port': 'esp8266', 'arch': 'xtensa', 'sysname': 'esp8266', 'release': '1.18', 'name': 'micropython', 'mpy': 9733, 'version': '1.18', 'machine': 'ESP module with ESP8266', 'build': '', 'nodename': 'esp8266', 'platform': 'esp8266', 'family': 'micropython'}
# Stubber: 1.5.4
from typing import Any

AP_IF = 1  # type: int
AUTH_OPEN = 0  # type: int
AUTH_WEP = 1  # type: int
AUTH_WPA2_PSK = 3  # type: int
AUTH_WPA_PSK = 2  # type: int
AUTH_WPA_WPA2_PSK = 4  # type: int
MODE_11B = 1  # type: int
MODE_11G = 2  # type: int
MODE_11N = 3  # type: int
STAT_CONNECTING = 1  # type: int
STAT_CONNECT_FAIL = 4  # type: int
STAT_GOT_IP = 5  # type: int
STAT_IDLE = 0  # type: int
STAT_NO_AP_FOUND = 3  # type: int
STAT_WRONG_PASSWORD = 2  # type: int
STA_IF = 0  # type: int


def WLAN(*args, **kwargs) -> Any:
    ...


def phy_mode(*args, **kwargs) -> Any:
    ...



from typing import Any

class Entry:
    time: Any
    obj: Any
    userdata: Any
    def __init__(self, time, obj, userdata) -> None: ...
    def __lt__(self, another): ...

class utimeq:
    heap: Any
    max_sz: Any
    def __init__(self, sz) -> None: ...
    def __bool__(self): ...
    def push(self, time, obj, userdata): ...
    def pop(self, arr) -> None: ...
    def peektime(self): ...
    def remove(self, id) -> None: ...



import sys
import os


class LS:
    def __repr__(self):
        self.__call__()
        return ""

    def __call__(self, path="."):
        l = os.listdir(path)
        l.sort()
        for f in l:
            st = os.stat("%s/%s" % (path, f))
            if st[0] & 0x4000:  # stat.S_IFDIR
                print("   <dir> %s" % f)
            else:
                print("% 8d %s" % (st[6], f))


class PWD:
    def __repr__(self):
        return os.getcwd()

    def __call__(self):
        return self.__repr__()


class CLEAR:
    def __repr__(self):
        return "\x1b[2J\x1b[H"

    def __call__(self):
        return self.__repr__()


pwd = PWD()
ls = LS()
clear = CLEAR()

cd = os.chdir
mkdir = os.mkdir
mv = os.rename
rm = os.remove
rmdir = os.rmdir


def head(f, n=10):
    with open(f) as f:
        for i in range(n):
            l = f.readline()
            if not l:
                break
            sys.stdout.write(l)


def cat(f):
    head(f, 1 << 30)


def newfile(path):
    print("Type file contents line by line, finish with EOF (Ctrl+D).")
    with open(path, "w") as f:
        while 1:
            try:
                l = input()
            except EOFError:
                break
            f.write(l)
            f.write("\n")


class Man:
    def __repr__(self):
        return """
upysh is intended to be imported using:
from upysh import *

To see this help text again, type "man".

upysh commands:
pwd, cd("new_dir"), ls, ls(...), head(...), cat(...)
newfile(...), mv("old", "new"), rm(...), mkdir(...), rmdir(...),
clear
"""


man = Man()

print(man)



try:
    import usocket as socket
except:
    import socket
try:
    import ustruct as struct
except:
    import struct

# (date(2000, 1, 1) - date(1900, 1, 1)).days * 24*60*60
NTP_DELTA = 3155673600

# The NTP host can be configured at runtime by doing: ntptime.host = 'myhost.org'
host = "pool.ntp.org"


def time():
    NTP_QUERY = bytearray(48)
    NTP_QUERY[0] = 0x1B
    addr = socket.getaddrinfo(host, 123)[0][-1]
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    try:
        s.settimeout(1)
        res = s.sendto(NTP_QUERY, addr)
        msg = s.recv(48)
    finally:
        s.close()
    val = struct.unpack("!I", msg[40:44])[0]
    return val - NTP_DELTA


# There's currently no timezone support in MicroPython, and the RTC is set in UTC time.
def settime():
    t = time()
    import machine
    import utime

    tm = utime.gmtime(t)
    machine.RTC().datetime((tm[0], tm[1], tm[2], tm[6] + 1, tm[3], tm[4], tm[5], 0))



"""
Module: 'machine' on micropython-v1.18-esp8266
"""
# MCU: {'ver': 'v1.18', 'port': 'esp8266', 'arch': 'xtensa', 'sysname': 'esp8266', 'release': '1.18', 'name': 'micropython', 'mpy': 9733, 'version': '1.18', 'machine': 'ESP module with ESP8266', 'build': '', 'nodename': 'esp8266', 'platform': 'esp8266', 'family': 'micropython'}
# Stubber: 1.5.4
from typing import Any


class ADC:
    """"""

    def __init__(self, *argv, **kwargs) -> None:
        """"""
        ...

    def read(self, *args, **kwargs) -> Any:
        ...

    def read_u16(self, *args, **kwargs) -> Any:
        ...


DEEPSLEEP = 4  # type: int
DEEPSLEEP_RESET = 5  # type: int
HARD_RESET = 6  # type: int


class I2C:
    """"""

    def __init__(self, *argv, **kwargs) -> None:
        """"""
        ...

    def readinto(self, *args, **kwargs) -> Any:
        ...

    def start(self, *args, **kwargs) -> Any:
        ...

    def stop(self, *args, **kwargs) -> Any:
        ...

    def write(self, *args, **kwargs) -> Any:
        ...

    def init(self, *args, **kwargs) -> Any:
        ...

    def readfrom(self, *args, **kwargs) -> Any:
        ...

    def readfrom_into(self, *args, **kwargs) -> Any:
        ...

    def readfrom_mem(self, *args, **kwargs) -> Any:
        ...

    def readfrom_mem_into(self, *args, **kwargs) -> Any:
        ...

    def scan(self, *args, **kwargs) -> Any:
        ...

    def writeto(self, *args, **kwargs) -> Any:
        ...

    def writeto_mem(self, *args, **kwargs) -> Any:
        ...

    def writevto(self, *args, **kwargs) -> Any:
        ...


class PWM:
    """"""

    def __init__(self, *argv, **kwargs) -> None:
        """"""
        ...

    def deinit(self, *args, **kwargs) -> Any:
        ...

    def duty(self, *args, **kwargs) -> Any:
        ...

    def freq(self, *args, **kwargs) -> Any:
        ...

    def init(self, *args, **kwargs) -> Any:
        ...


PWRON_RESET = 0  # type: int


class Pin:
    """"""

    def __init__(self, *argv, **kwargs) -> None:
        """"""
        ...

    def value(self, *args, **kwargs) -> Any:
        ...

    IN = 0  # type: int
    IRQ_FALLING = 2  # type: int
    IRQ_RISING = 1  # type: int
    OPEN_DRAIN = 2  # type: int
    OUT = 1  # type: int
    PULL_UP = 1  # type: int

    def init(self, *args, **kwargs) -> Any:
        ...

    def irq(self, *args, **kwargs) -> Any:
        ...

    def off(self, *args, **kwargs) -> Any:
        ...

    def on(self, *args, **kwargs) -> Any:
        ...


class RTC:
    """"""

    def __init__(self, *argv, **kwargs) -> None:
        """"""
        ...

    ALARM0 = 0  # type: int

    def alarm(self, *args, **kwargs) -> Any:
        ...

    def alarm_left(self, *args, **kwargs) -> Any:
        ...

    def datetime(self, *args, **kwargs) -> Any:
        ...

    def irq(self, *args, **kwargs) -> Any:
        ...

    def memory(self, *args, **kwargs) -> Any:
        ...


SOFT_RESET = 4  # type: int


class SPI:
    """"""

    def __init__(self, *argv, **kwargs) -> None:
        """"""
        ...

    def read(self, *args, **kwargs) -> Any:
        ...

    def readinto(self, *args, **kwargs) -> Any:
        ...

    def write(self, *args, **kwargs) -> Any:
        ...

    LSB = 1  # type: int
    MSB = 0  # type: int

    def deinit(self, *args, **kwargs) -> Any:
        ...

    def init(self, *args, **kwargs) -> Any:
        ...

    def write_readinto(self, *args, **kwargs) -> Any:
        ...


class Signal:
    """"""

    def __init__(self, *argv, **kwargs) -> None:
        """"""
        ...

    def value(self, *args, **kwargs) -> Any:
        ...

    def off(self, *args, **kwargs) -> Any:
        ...

    def on(self, *args, **kwargs) -> Any:
        ...


class SoftI2C:
    """"""

    def __init__(self, *argv, **kwargs) -> None:
        """"""
        ...

    def readinto(self, *args, **kwargs) -> Any:
        ...

    def start(self, *args, **kwargs) -> Any:
        ...

    def stop(self, *args, **kwargs) -> Any:
        ...

    def write(self, *args, **kwargs) -> Any:
        ...

    def init(self, *args, **kwargs) -> Any:
        ...

    def readfrom(self, *args, **kwargs) -> Any:
        ...

    def readfrom_into(self, *args, **kwargs) -> Any:
        ...

    def readfrom_mem(self, *args, **kwargs) -> Any:
        ...

    def readfrom_mem_into(self, *args, **kwargs) -> Any:
        ...

    def scan(self, *args, **kwargs) -> Any:
        ...

    def writeto(self, *args, **kwargs) -> Any:
        ...

    def writeto_mem(self, *args, **kwargs) -> Any:
        ...

    def writevto(self, *args, **kwargs) -> Any:
        ...


class SoftSPI:
    """"""

    def __init__(self, *argv, **kwargs) -> None:
        """"""
        ...

    def read(self, *args, **kwargs) -> Any:
        ...

    def readinto(self, *args, **kwargs) -> Any:
        ...

    def write(self, *args, **kwargs) -> Any:
        ...

    LSB = 1  # type: int
    MSB = 0  # type: int

    def deinit(self, *args, **kwargs) -> Any:
        ...

    def init(self, *args, **kwargs) -> Any:
        ...

    def write_readinto(self, *args, **kwargs) -> Any:
        ...


class Timer:
    """"""

    def __init__(self, *argv, **kwargs) -> None:
        """"""
        ...

    ONE_SHOT = 0  # type: int
    PERIODIC = 1  # type: int

    def deinit(self, *args, **kwargs) -> Any:
        ...

    def init(self, *args, **kwargs) -> Any:
        ...


class UART:
    """"""

    def __init__(self, *argv, **kwargs) -> None:
        """"""
        ...

    def any(self, *args, **kwargs) -> Any:
        ...

    def read(self, *args, **kwargs) -> Any:
        ...

    def readinto(self, *args, **kwargs) -> Any:
        ...

    def readline(self, *args, **kwargs) -> Any:
        ...

    def write(self, *args, **kwargs) -> Any:
        ...

    def init(self, *args, **kwargs) -> Any:
        ...


class WDT:
    """"""

    def __init__(self, *argv, **kwargs) -> None:
        """"""
        ...

    def deinit(self, *args, **kwargs) -> Any:
        ...

    def feed(self, *args, **kwargs) -> Any:
        ...


WDT_RESET = 1  # type: int


def bitstream(*args, **kwargs) -> Any:
    ...


def deepsleep(*args, **kwargs) -> Any:
    ...


def disable_irq(*args, **kwargs) -> Any:
    ...


def enable_irq(*args, **kwargs) -> Any:
    ...


def freq(*args, **kwargs) -> Any:
    ...


def idle(*args, **kwargs) -> Any:
    ...


def lightsleep(*args, **kwargs) -> Any:
    ...


mem16: Any  ## <class 'mem'> = <16-bit memory>
mem32: Any  ## <class 'mem'> = <32-bit memory>
mem8: Any  ## <class 'mem'> = <8-bit memory>


def reset(*args, **kwargs) -> Any:
    ...


def reset_cause(*args, **kwargs) -> Any:
    ...


def sleep(*args, **kwargs) -> Any:
    ...


def soft_reset(*args, **kwargs) -> Any:
    ...


def time_pulse_us(*args, **kwargs) -> Any:
    ...


def unique_id(*args, **kwargs) -> Any:
    ...



from typing import Any

AF_INET: int
AF_INET6: int
IPPROTO_IP: int
IP_ADD_MEMBERSHIP: int
SOCK_DGRAM: int
SOCK_RAW: int
SOCK_STREAM: int
SOL_SOCKET: int
SO_REUSEADDR: int

def callback(*args, **kwargs) -> Any: ...
def getaddrinfo(*args, **kwargs) -> Any: ...
def print_pcbs(*args, **kwargs) -> Any: ...
def reset(*args, **kwargs) -> Any: ...

class socket:
    def __init__(self, *argv, **kwargs) -> None: ...
    def close(self, *args, **kwargs) -> Any: ...
    def read(self, *args, **kwargs) -> Any: ...
    def readinto(self, *args, **kwargs) -> Any: ...
    def readline(self, *args, **kwargs) -> Any: ...
    def send(self, *args, **kwargs) -> Any: ...
    def write(self, *args, **kwargs) -> Any: ...
    def accept(self, *args, **kwargs) -> Any: ...
    def bind(self, *args, **kwargs) -> Any: ...
    def connect(self, *args, **kwargs) -> Any: ...
    def listen(self, *args, **kwargs) -> Any: ...
    def makefile(self, *args, **kwargs) -> Any: ...
    def recv(self, *args, **kwargs) -> Any: ...
    def recvfrom(self, *args, **kwargs) -> Any: ...
    def sendall(self, *args, **kwargs) -> Any: ...
    def sendto(self, *args, **kwargs) -> Any: ...
    def setblocking(self, *args, **kwargs) -> Any: ...
    def setsockopt(self, *args, **kwargs) -> Any: ...
    def settimeout(self, *args, **kwargs) -> Any: ...



from typing import Any

class Response:
    raw: Any
    encoding: str
    _cached: Any
    def __init__(self, f) -> None: ...
    def close(self) -> None: ...
    @property
    def content(self): ...
    @property
    def text(self): ...
    def json(self): ...

def request(method, url, data: Any | None = ..., json: Any | None = ..., headers=..., stream: Any | None = ...): ...
def head(url, **kw): ...
def get(url, **kw): ...
def post(url, **kw): ...
def put(url, **kw): ...
def patch(url, **kw): ...
def delete(url, **kw): ...



from typing import Any

class NeoPixel:
    ORDER: Any
    pin: Any
    n: Any
    bpp: Any
    buf: Any
    timing: Any
    def __init__(self, pin, n, bpp: int = ..., timing: int = ...) -> None: ...
    def __len__(self): ...
    def __setitem__(self, i, v) -> None: ...
    def __getitem__(self, i): ...
    def fill(self, v) -> None: ...
    def write(self) -> None: ...



import os
from typing import Any

class LS:
    def __repr__(self): ...
    def __call__(self, path: str = ...) -> None: ...

class PWD:
    def __repr__(self): ...
    def __call__(self): ...

class CLEAR:
    def __repr__(self): ...
    def __call__(self): ...

pwd: Any
ls: Any
clear: Any
cd = os.chdir
mkdir = os.mkdir
mv = os.rename
rm = os.remove
rmdir = os.rmdir

def head(f, n: int = ...) -> None: ...
def cat(f) -> None: ...
def newfile(path) -> None: ...

class Man:
    def __repr__(self): ...

man: Any



from zlib import *
from typing import Any

class DecompIO:
    stream: Any
    decomp: Any
    pending: bytes
    def __init__(self, stream, dict_bits, dictbuf: Any | None = ...) -> None: ...
    def read(self, size): ...



from typing import Any

def collect(*args, **kwargs) -> Any: ...
def disable(*args, **kwargs) -> Any: ...
def enable(*args, **kwargs) -> Any: ...
def isenabled(*args, **kwargs) -> Any: ...
def mem_alloc(*args, **kwargs) -> Any: ...
def mem_free(*args, **kwargs) -> Any: ...
def threshold(*args, **kwargs) -> Any: ...



from typing import Any

argv: list
byteorder: str

def exit(*args, **kwargs) -> Any: ...

implementation: tuple
maxsize: int
modules: dict
path: list
platform: str

def print_exception(*args, **kwargs) -> Any: ...

stderr: Any
stdin: Any
stdout: Any
version: str
version_info: tuple



import framebuf
from typing import Any

SET_CONTRAST: Any
SET_ENTIRE_ON: Any
SET_NORM_INV: Any
SET_DISP: Any
SET_MEM_ADDR: Any
SET_COL_ADDR: Any
SET_PAGE_ADDR: Any
SET_DISP_START_LINE: Any
SET_SEG_REMAP: Any
SET_MUX_RATIO: Any
SET_IREF_SELECT: Any
SET_COM_OUT_DIR: Any
SET_DISP_OFFSET: Any
SET_COM_PIN_CFG: Any
SET_DISP_CLK_DIV: Any
SET_PRECHARGE: Any
SET_VCOM_DESEL: Any
SET_CHARGE_PUMP: Any

class SSD1306(framebuf.FrameBuffer):
    width: Any
    height: Any
    external_vcc: Any
    pages: Any
    buffer: Any
    def __init__(self, width, height, external_vcc) -> None: ...
    def init_display(self) -> None: ...
    def poweroff(self) -> None: ...
    def poweron(self) -> None: ...
    def contrast(self, contrast) -> None: ...
    def invert(self, invert) -> None: ...
    def rotate(self, rotate) -> None: ...
    def show(self) -> None: ...

class SSD1306_I2C(SSD1306):
    i2c: Any
    addr: Any
    temp: Any
    write_list: Any
    def __init__(self, width, height, i2c, addr: int = ..., external_vcc: bool = ...) -> None: ...
    def write_cmd(self, cmd) -> None: ...
    def write_data(self, buf) -> None: ...

class SSD1306_SPI(SSD1306):
    rate: Any
    spi: Any
    dc: Any
    res: Any
    cs: Any
    def __init__(self, width, height, spi, dc, res, cs, external_vcc: bool = ...) -> None: ...
    def write_cmd(self, cmd) -> None: ...
    def write_data(self, buf) -> None: ...



from typing import Any

debug: bool
index_urls: Any
install_path: Any
cleanup_files: Any
gzdict_sz: Any
file_buf: Any

class NotFoundError(Exception): ...

def op_split(path): ...
def op_basename(path): ...
def _makedirs(name, mode: int = ...): ...
def save_file(fname, subf) -> None: ...
def install_tar(f, prefix): ...
def expandhome(s): ...

warn_ussl: bool

def url_open(url): ...
def get_pkg_metadata(name): ...
def fatal(msg, exc: Any | None = ...) -> None: ...
def install_pkg(pkg_spec, install_path): ...
def install(to_install, install_path: Any | None = ...) -> None: ...
def get_install_path(): ...
def cleanup() -> None: ...
def help() -> None: ...
def main() -> None: ...



"""
Module: 'ucryptolib' on micropython-v1.18-esp8266
"""
# MCU: {'ver': 'v1.18', 'port': 'esp8266', 'arch': 'xtensa', 'sysname': 'esp8266', 'release': '1.18', 'name': 'micropython', 'mpy': 9733, 'version': '1.18', 'machine': 'ESP module with ESP8266', 'build': '', 'nodename': 'esp8266', 'platform': 'esp8266', 'family': 'micropython'}
# Stubber: 1.5.4
from typing import Any


class aes:
    """"""

    def __init__(self, *argv, **kwargs) -> None:
        """"""
        ...

    def decrypt(self, *args, **kwargs) -> Any:
        ...

    def encrypt(self, *args, **kwargs) -> Any:
        ...



# (c) 2019 Paul Sokolovsky. MIT license.
import ctypes
import array


class Func:
    def __init__(self, f, restype):
        self.f = f
        self.restype = restype

    def __call__(self, *args):
        def conv_arg(x):
            if isinstance(x, str):
                return x.encode()
            if isinstance(x, (bytearray, array.array)):
                a = ctypes.c_byte.from_buffer(x)
                return ctypes.addressof(a)
            return x

        args = [conv_arg(x) for x in args]
        # print(args)
        res = self.f(*args)
        if self.restype == "s":
            res = res.decode("utf-8")
        return res


class Var:
    def __init__(self, v):
        self.v = v

    def get(self):
        return self.v.value


class DynMod:

    typemap = {
        "v": None,
        "s": ctypes.c_char_p,
        "S": ctypes.c_char_p,
        "i": ctypes.c_int,
        "I": ctypes.c_uint,
        "l": ctypes.c_long,
        "L": ctypes.c_ulong,
        "q": ctypes.c_longlong,
        "Q": ctypes.c_ulonglong,
        "p": ctypes.c_void_p,
        "P": ctypes.c_void_p,
        "C": ctypes.c_void_p,
    }

    def __init__(self, name):
        self.mod = ctypes.CDLL(name)

    def func(self, ret, name, params):
        f = getattr(self.mod, name)
        argtypes = []
        for p in params:
            argtypes.append(self.typemap[p])
        # print(name, argtypes)
        f.argtypes = argtypes
        f.restype = self.typemap[ret]
        return Func(f, ret)

    def var(self, type, name):
        ctype = self.typemap[type]
        v = ctype.in_dll(self.mod, name)
        return Var(v)


def open(name):
    return DynMod(name)


def func(ret, addr, params):
    types = [DynMod.typemap[ret]]
    for p in params:
        types.append(DynMod.typemap[p])
    ftype = ctypes.CFUNCTYPE(*types)
    return ftype(addr)


def callback(ret, func, params):
    types = [DynMod.typemap[ret]]
    for p in params:
        types.append(DynMod.typemap[p])
    ftype = ctypes.CFUNCTYPE(*types)
    return ftype(func)



"""
Module: 'os' on micropython-v1.18-esp8266
"""
# MCU: {'ver': 'v1.18', 'port': 'esp8266', 'arch': 'xtensa', 'sysname': 'esp8266', 'release': '1.18', 'name': 'micropython', 'mpy': 9733, 'version': '1.18', 'machine': 'ESP module with ESP8266', 'build': '', 'nodename': 'esp8266', 'platform': 'esp8266', 'family': 'micropython'}
# Stubber: 1.5.4
from typing import Any


def remove(*args, **kwargs) -> Any:
    ...


class VfsFat:
    """"""

    def __init__(self, *argv, **kwargs) -> None:
        """"""
        ...

    def open(self, *args, **kwargs) -> Any:
        ...

    def remove(self, *args, **kwargs) -> Any:
        ...

    def chdir(self, *args, **kwargs) -> Any:
        ...

    def getcwd(self, *args, **kwargs) -> Any:
        ...

    def ilistdir(self, *args, **kwargs) -> Any:
        ...

    def mkdir(self, *args, **kwargs) -> Any:
        ...

    def mkfs(self, *args, **kwargs) -> Any:
        ...

    def mount(self, *args, **kwargs) -> Any:
        ...

    def rename(self, *args, **kwargs) -> Any:
        ...

    def rmdir(self, *args, **kwargs) -> Any:
        ...

    def stat(self, *args, **kwargs) -> Any:
        ...

    def statvfs(self, *args, **kwargs) -> Any:
        ...

    def umount(self, *args, **kwargs) -> Any:
        ...


class VfsLfs2:
    """"""

    def __init__(self, *argv, **kwargs) -> None:
        """"""
        ...

    def open(self, *args, **kwargs) -> Any:
        ...

    def remove(self, *args, **kwargs) -> Any:
        ...

    def chdir(self, *args, **kwargs) -> Any:
        ...

    def getcwd(self, *args, **kwargs) -> Any:
        ...

    def ilistdir(self, *args, **kwargs) -> Any:
        ...

    def mkdir(self, *args, **kwargs) -> Any:
        ...

    def mkfs(self, *args, **kwargs) -> Any:
        ...

    def mount(self, *args, **kwargs) -> Any:
        ...

    def rename(self, *args, **kwargs) -> Any:
        ...

    def rmdir(self, *args, **kwargs) -> Any:
        ...

    def stat(self, *args, **kwargs) -> Any:
        ...

    def statvfs(self, *args, **kwargs) -> Any:
        ...

    def umount(self, *args, **kwargs) -> Any:
        ...


def chdir(*args, **kwargs) -> Any:
    ...


def dupterm(*args, **kwargs) -> Any:
    ...


def dupterm_notify(*args, **kwargs) -> Any:
    ...


def getcwd(*args, **kwargs) -> Any:
    ...


def ilistdir(*args, **kwargs) -> Any:
    ...


def listdir(*args, **kwargs) -> Any:
    ...


def mkdir(*args, **kwargs) -> Any:
    ...


def mount(*args, **kwargs) -> Any:
    ...


def rename(*args, **kwargs) -> Any:
    ...


def rmdir(*args, **kwargs) -> Any:
    ...


def stat(*args, **kwargs) -> Any:
    ...


def statvfs(*args, **kwargs) -> Any:
    ...


def umount(*args, **kwargs) -> Any:
    ...


def uname(*args, **kwargs) -> Any:
    ...


def urandom(*args, **kwargs) -> Any:
    ...



"""
Module: 'socket' on micropython-v1.18-esp8266
"""
# MCU: {'ver': 'v1.18', 'port': 'esp8266', 'arch': 'xtensa', 'sysname': 'esp8266', 'release': '1.18', 'name': 'micropython', 'mpy': 9733, 'version': '1.18', 'machine': 'ESP module with ESP8266', 'build': '', 'nodename': 'esp8266', 'platform': 'esp8266', 'family': 'micropython'}
# Stubber: 1.5.4
from typing import Any

AF_INET = 2  # type: int
AF_INET6 = 10  # type: int
IPPROTO_IP = 0  # type: int
IP_ADD_MEMBERSHIP = 1024  # type: int
SOCK_DGRAM = 2  # type: int
SOCK_RAW = 3  # type: int
SOCK_STREAM = 1  # type: int
SOL_SOCKET = 1  # type: int
SO_REUSEADDR = 4  # type: int


def callback(*args, **kwargs) -> Any:
    ...


def getaddrinfo(*args, **kwargs) -> Any:
    ...


def print_pcbs(*args, **kwargs) -> Any:
    ...


def reset(*args, **kwargs) -> Any:
    ...


class socket:
    """"""

    def __init__(self, *argv, **kwargs) -> None:
        """"""
        ...

    def close(self, *args, **kwargs) -> Any:
        ...

    def read(self, *args, **kwargs) -> Any:
        ...

    def readinto(self, *args, **kwargs) -> Any:
        ...

    def readline(self, *args, **kwargs) -> Any:
        ...

    def send(self, *args, **kwargs) -> Any:
        ...

    def write(self, *args, **kwargs) -> Any:
        ...

    def accept(self, *args, **kwargs) -> Any:
        ...

    def bind(self, *args, **kwargs) -> Any:
        ...

    def connect(self, *args, **kwargs) -> Any:
        ...

    def listen(self, *args, **kwargs) -> Any:
        ...

    def makefile(self, *args, **kwargs) -> Any:
        ...

    def recv(self, *args, **kwargs) -> Any:
        ...

    def recvfrom(self, *args, **kwargs) -> Any:
        ...

    def sendall(self, *args, **kwargs) -> Any:
        ...

    def sendto(self, *args, **kwargs) -> Any:
        ...

    def setblocking(self, *args, **kwargs) -> Any:
        ...

    def setsockopt(self, *args, **kwargs) -> Any:
        ...

    def settimeout(self, *args, **kwargs) -> Any:
        ...



from typing import Any

class array:
    def __init__(self, *argv, **kwargs) -> None: ...
    def append(self, *args, **kwargs) -> Any: ...
    def extend(self, *args, **kwargs) -> Any: ...
    def decode(self, *args, **kwargs) -> Any: ...



from typing import Any

AP_IF: int
AUTH_OPEN: int
AUTH_WEP: int
AUTH_WPA2_PSK: int
AUTH_WPA_PSK: int
AUTH_WPA_WPA2_PSK: int
MODE_11B: int
MODE_11G: int
MODE_11N: int
STAT_CONNECTING: int
STAT_CONNECT_FAIL: int
STAT_GOT_IP: int
STAT_IDLE: int
STAT_NO_AP_FOUND: int
STAT_WRONG_PASSWORD: int
STA_IF: int

def WLAN(*args, **kwargs) -> Any: ...
def phy_mode(*args, **kwargs) -> Any: ...



from typing import Any

class sha1:
    def __init__(self, *argv, **kwargs) -> None: ...
    def update(self, *args, **kwargs) -> Any: ...
    def digest(self, *args, **kwargs) -> Any: ...

class sha256:
    def __init__(self, *argv, **kwargs) -> None: ...
    def update(self, *args, **kwargs) -> Any: ...
    def digest(self, *args, **kwargs) -> Any: ...



"""
Module: 'esp' on micropython-v1.18-esp8266
"""
# MCU: {'ver': 'v1.18', 'port': 'esp8266', 'arch': 'xtensa', 'sysname': 'esp8266', 'release': '1.18', 'name': 'micropython', 'mpy': 9733, 'version': '1.18', 'machine': 'ESP module with ESP8266', 'build': '', 'nodename': 'esp8266', 'platform': 'esp8266', 'family': 'micropython'}
# Stubber: 1.5.4
from typing import Any

SLEEP_LIGHT = 1  # type: int
SLEEP_MODEM = 2  # type: int
SLEEP_NONE = 0  # type: int


def apa102_write(*args, **kwargs) -> Any:
    ...


def check_fw(*args, **kwargs) -> Any:
    ...


def deepsleep(*args, **kwargs) -> Any:
    ...


def dht_readinto(*args, **kwargs) -> Any:
    ...


def esf_free_bufs(*args, **kwargs) -> Any:
    ...


def flash_erase(*args, **kwargs) -> Any:
    ...


def flash_id(*args, **kwargs) -> Any:
    ...


def flash_read(*args, **kwargs) -> Any:
    ...


def flash_size(*args, **kwargs) -> Any:
    ...


def flash_user_start(*args, **kwargs) -> Any:
    ...


def flash_write(*args, **kwargs) -> Any:
    ...


def free(*args, **kwargs) -> Any:
    ...


def freemem(*args, **kwargs) -> Any:
    ...


def info(*args, **kwargs) -> Any:
    ...


def malloc(*args, **kwargs) -> Any:
    ...


def meminfo(*args, **kwargs) -> Any:
    ...


def osdebug(*args, **kwargs) -> Any:
    ...


def set_native_code_location(*args, **kwargs) -> Any:
    ...


def sleep_type(*args, **kwargs) -> Any:
    ...



"""
Module: 'errno' on micropython-v1.18-esp8266
"""
# MCU: {'ver': 'v1.18', 'port': 'esp8266', 'arch': 'xtensa', 'sysname': 'esp8266', 'release': '1.18', 'name': 'micropython', 'mpy': 9733, 'version': '1.18', 'machine': 'ESP module with ESP8266', 'build': '', 'nodename': 'esp8266', 'platform': 'esp8266', 'family': 'micropython'}
# Stubber: 1.5.4
from typing import Any

EACCES = 13  # type: int
EADDRINUSE = 98  # type: int
EAGAIN = 11  # type: int
EALREADY = 114  # type: int
EBADF = 9  # type: int
ECONNABORTED = 103  # type: int
ECONNREFUSED = 111  # type: int
ECONNRESET = 104  # type: int
EEXIST = 17  # type: int
EHOSTUNREACH = 113  # type: int
EINPROGRESS = 115  # type: int
EINVAL = 22  # type: int
EIO = 5  # type: int
EISDIR = 21  # type: int
ENOBUFS = 105  # type: int
ENODEV = 19  # type: int
ENOENT = 2  # type: int
ENOMEM = 12  # type: int
ENOTCONN = 107  # type: int
EOPNOTSUPP = 95  # type: int
EPERM = 1  # type: int
ETIMEDOUT = 110  # type: int
errorcode = {}  # type: dict



from typing import Any

SLEEP_LIGHT: int
SLEEP_MODEM: int
SLEEP_NONE: int

def apa102_write(*args, **kwargs) -> Any: ...
def check_fw(*args, **kwargs) -> Any: ...
def deepsleep(*args, **kwargs) -> Any: ...
def dht_readinto(*args, **kwargs) -> Any: ...
def esf_free_bufs(*args, **kwargs) -> Any: ...
def flash_erase(*args, **kwargs) -> Any: ...
def flash_id(*args, **kwargs) -> Any: ...
def flash_read(*args, **kwargs) -> Any: ...
def flash_size(*args, **kwargs) -> Any: ...
def flash_user_start(*args, **kwargs) -> Any: ...
def flash_write(*args, **kwargs) -> Any: ...
def free(*args, **kwargs) -> Any: ...
def freemem(*args, **kwargs) -> Any: ...
def info(*args, **kwargs) -> Any: ...
def malloc(*args, **kwargs) -> Any: ...
def meminfo(*args, **kwargs) -> Any: ...
def osdebug(*args, **kwargs) -> Any: ...
def set_native_code_location(*args, **kwargs) -> Any: ...
def sleep_type(*args, **kwargs) -> Any: ...



from typing import Any

def calcsize(*args, **kwargs) -> Any: ...
def pack(*args, **kwargs) -> Any: ...
def pack_into(*args, **kwargs) -> Any: ...
def unpack(*args, **kwargs) -> Any: ...
def unpack_from(*args, **kwargs) -> Any: ...



import collections
from collections import OrderedDict, namedtuple


class deque:
    def __init__(self, iterable, maxlen, flags=0):
        assert iterable == ()
        self.maxlen = maxlen
        self.flags = flags
        self.d = collections.deque(iterable, maxlen)

    def __len__(self):
        return len(self.d)

    def append(self, x):
        if self.flags & 1:
            if len(self.d) == self.maxlen:
                raise IndexError
        self.d.append(x)

    def popleft(self):
        return self.d.popleft()



from typing import Any

def gmtime(*args, **kwargs) -> Any: ...
def localtime(*args, **kwargs) -> Any: ...
def mktime(*args, **kwargs) -> Any: ...
def sleep(*args, **kwargs) -> Any: ...
def sleep_ms(*args, **kwargs) -> Any: ...
def sleep_us(*args, **kwargs) -> Any: ...
def ticks_add(*args, **kwargs) -> Any: ...
def ticks_cpu(*args, **kwargs) -> Any: ...
def ticks_diff(*args, **kwargs) -> Any: ...
def ticks_ms(*args, **kwargs) -> Any: ...
def ticks_us(*args, **kwargs) -> Any: ...
def time(*args, **kwargs) -> Any: ...
def time_ns(*args, **kwargs) -> Any: ...



# This module should be imported from REPL, not run from command line.
import socket
import uos
import network
import uwebsocket
import websocket_helper
import _webrepl

listen_s = None
client_s = None


def setup_conn(port, accept_handler):
    global listen_s
    listen_s = socket.socket()
    listen_s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)

    ai = socket.getaddrinfo("0.0.0.0", port)
    addr = ai[0][4]

    listen_s.bind(addr)
    listen_s.listen(1)
    if accept_handler:
        listen_s.setsockopt(socket.SOL_SOCKET, 20, accept_handler)
    for i in (network.AP_IF, network.STA_IF):
        iface = network.WLAN(i)
        if iface.active():
            print("WebREPL daemon started on ws://%s:%d" % (iface.ifconfig()[0], port))
    return listen_s


def accept_conn(listen_sock):
    global client_s
    cl, remote_addr = listen_sock.accept()
    prev = uos.dupterm(None)
    uos.dupterm(prev)
    if prev:
        print("\nConcurrent WebREPL connection from", remote_addr, "rejected")
        cl.close()
        return
    print("\nWebREPL connection from:", remote_addr)
    client_s = cl
    websocket_helper.server_handshake(cl)
    ws = uwebsocket.websocket(cl, True)
    ws = _webrepl._webrepl(ws)
    cl.setblocking(False)
    # notify REPL on socket incoming data (ESP32/ESP8266-only)
    if hasattr(uos, "dupterm_notify"):
        cl.setsockopt(socket.SOL_SOCKET, 20, uos.dupterm_notify)
    uos.dupterm(ws)


def stop():
    global listen_s, client_s
    uos.dupterm(None)
    if client_s:
        client_s.close()
    if listen_s:
        listen_s.close()


def start(port=8266, password=None):
    stop()
    if password is None:
        try:
            import webrepl_cfg

            _webrepl.password(webrepl_cfg.PASS)
            setup_conn(port, accept_conn)
            print("Started webrepl in normal mode")
        except:
            print("WebREPL is not configured, run 'import webrepl_setup'")
    else:
        _webrepl.password(password)
        setup_conn(port, accept_conn)
        print("Started webrepl in manual override mode")


def start_foreground(port=8266):
    stop()
    s = setup_conn(port, None)
    accept_conn(s)



"""
Module: 'time' on micropython-v1.18-esp8266
"""
# MCU: {'ver': 'v1.18', 'port': 'esp8266', 'arch': 'xtensa', 'sysname': 'esp8266', 'release': '1.18', 'name': 'micropython', 'mpy': 9733, 'version': '1.18', 'machine': 'ESP module with ESP8266', 'build': '', 'nodename': 'esp8266', 'platform': 'esp8266', 'family': 'micropython'}
# Stubber: 1.5.4
from typing import Any


def gmtime(*args, **kwargs) -> Any:
    ...


def localtime(*args, **kwargs) -> Any:
    ...


def mktime(*args, **kwargs) -> Any:
    ...


def sleep(*args, **kwargs) -> Any:
    ...


def sleep_ms(*args, **kwargs) -> Any:
    ...


def sleep_us(*args, **kwargs) -> Any:
    ...


def ticks_add(*args, **kwargs) -> Any:
    ...


def ticks_cpu(*args, **kwargs) -> Any:
    ...


def ticks_diff(*args, **kwargs) -> Any:
    ...


def ticks_ms(*args, **kwargs) -> Any:
    ...


def ticks_us(*args, **kwargs) -> Any:
    ...


def time(*args, **kwargs) -> Any:
    ...


def time_ns(*args, **kwargs) -> Any:
    ...



from typing import Any

listen_s: Any
client_s: Any

def setup_conn(port, accept_handler): ...
def accept_conn(listen_sock) -> None: ...
def stop() -> None: ...
def start(port: int = ..., password: Any | None = ...) -> None: ...
def start_foreground(port: int = ...) -> None: ...



import usocket


class Response:
    def __init__(self, f):
        self.raw = f
        self.encoding = "utf-8"
        self._cached = None

    def close(self):
        if self.raw:
            self.raw.close()
            self.raw = None
        self._cached = None

    @property
    def content(self):
        if self._cached is None:
            try:
                self._cached = self.raw.read()
            finally:
                self.raw.close()
                self.raw = None
        return self._cached

    @property
    def text(self):
        return str(self.content, self.encoding)

    def json(self):
        import ujson

        return ujson.loads(self.content)


def request(method, url, data=None, json=None, headers={}, stream=None):
    try:
        proto, dummy, host, path = url.split("/", 3)
    except ValueError:
        proto, dummy, host = url.split("/", 2)
        path = ""
    if proto == "http:":
        port = 80
    elif proto == "https:":
        import ussl

        port = 443
    else:
        raise ValueError("Unsupported protocol: " + proto)

    if ":" in host:
        host, port = host.split(":", 1)
        port = int(port)

    ai = usocket.getaddrinfo(host, port, 0, usocket.SOCK_STREAM)
    ai = ai[0]

    s = usocket.socket(ai[0], ai[1], ai[2])
    try:
        s.connect(ai[-1])
        if proto == "https:":
            s = ussl.wrap_socket(s, server_hostname=host)
        s.write(b"%s /%s HTTP/1.0\r\n" % (method, path))
        if not "Host" in headers:
            s.write(b"Host: %s\r\n" % host)
        # Iterate over keys to avoid tuple alloc
        for k in headers:
            s.write(k)
            s.write(b": ")
            s.write(headers[k])
            s.write(b"\r\n")
        if json is not None:
            assert data is None
            import ujson

            data = ujson.dumps(json)
            s.write(b"Content-Type: application/json\r\n")
        if data:
            s.write(b"Content-Length: %d\r\n" % len(data))
        s.write(b"\r\n")
        if data:
            s.write(data)

        l = s.readline()
        # print(l)
        l = l.split(None, 2)
        status = int(l[1])
        reason = ""
        if len(l) > 2:
            reason = l[2].rstrip()
        while True:
            l = s.readline()
            if not l or l == b"\r\n":
                break
            # print(l)
            if l.startswith(b"Transfer-Encoding:"):
                if b"chunked" in l:
                    raise ValueError("Unsupported " + l)
            elif l.startswith(b"Location:") and not 200 <= status <= 299:
                raise NotImplementedError("Redirects not yet supported")
    except OSError:
        s.close()
        raise

    resp = Response(s)
    resp.status_code = status
    resp.reason = reason
    return resp


def head(url, **kw):
    return request("HEAD", url, **kw)


def get(url, **kw):
    return request("GET", url, **kw)


def post(url, **kw):
    return request("POST", url, **kw)


def put(url, **kw):
    return request("PUT", url, **kw)


def patch(url, **kw):
    return request("PATCH", url, **kw)


def delete(url, **kw):
    return request("DELETE", url, **kw)



from typing import Any

class _hash:
    _: Any
    def __init__(self, data: Any | None = ...) -> None: ...
    def update(self, data) -> None: ...
    def digest(self): ...

class md5(_hash): ...
class sha1(_hash): ...
class sha256(_hash): ...



# (c) 2019 Paul Sokolovsky, MIT license
# This module is part of the Pycopy project, https://github.com/pfalcon/pycopy
from token import *
from ucollections import namedtuple


COMMENT = N_TOKENS + 0
NL = N_TOKENS + 1
ENCODING = N_TOKENS + 2
tok_name[COMMENT] = "COMMENT"
tok_name[NL] = "NL"
tok_name[ENCODING] = "ENCODING"


class TokenInfo(namedtuple("TokenInfo", ("type", "string", "start", "end", "line"))):
    def __str__(self):
        return "TokenInfo(type=%d (%s), string=%r, startl=%d, line=%r)" % (
            self.type,
            tok_name[self.type],
            self.string,
            self.start,
            self.line,
        )


def get_indent(l):
    for i in range(len(l)):
        if l[i] != " ":
            return i, l[i:]


def get_str(l, readline):
    lineno = 0

    if l.startswith('"""') or l.startswith("'''"):
        s = sep = l[0:3]
        l = l[3:]
        pos = 0
        while True:
            i = l.find(sep, pos)
            if i >= 0:
                if i > 0 and l[i - 1] == "\\":
                    pos = i + 1
                    continue
                break
            s += l
            l = readline()
            pos = 0
            assert l
            lineno += 1
        s += l[: i + 3]
        return s, l[i + 3 :], lineno

    s = sep = l[0]
    l = l[1:]
    quoted = False
    while l:
        c = l[0]
        l = l[1:]
        s += c
        if quoted:
            quoted = False
        elif c == "\\":
            if l == "\n":
                s += "\n"
                l = readline()
                lineno += 1
                continue
            quoted = True
        elif c == sep:
            break
    return s, l, lineno


def tokenize(readline):

    indent_stack = [0]
    lineno = 0
    paren_level = 0

    yield TokenInfo(ENCODING, "utf-8", 0, 0, "")

    while True:
        l = readline()
        lineno += 1
        org_l = l
        if not l:
            break
        i, l = get_indent(l)

        if l == "\n":
            yield TokenInfo(NL, l, lineno, 0, org_l)
            continue
        elif l == "\x0c\n":
            yield TokenInfo(NL, "\n", lineno, 0, org_l)
            continue

        if l.startswith("#"):
            yield TokenInfo(COMMENT, l.rstrip("\n"), lineno, 0, org_l)
            yield TokenInfo(NL, "\n", lineno, 0, org_l)
            continue

        if paren_level == 0:
            if i > indent_stack[-1]:
                yield TokenInfo(INDENT, " " * i, lineno, 0, org_l)
                indent_stack.append(i)
            elif i < indent_stack[-1]:
                while i != indent_stack[-1]:
                    yield TokenInfo(DEDENT, "", lineno, 0, org_l)
                    indent_stack.pop()

        while l:
            if l[0].isdigit() or (l.startswith(".") and len(l) > 1 and l[1].isdigit()):
                seen_dot = False
                t = ""
                if l.startswith("0x") or l.startswith("0X"):
                    t = "0x"
                    l = l[2:]
                elif l.startswith("0o") or l.startswith("0O"):
                    t = "0o"
                    l = l[2:]
                elif l.startswith("0b") or l.startswith("0B"):
                    t = "0b"
                    l = l[2:]
                while l and (l[0].isdigit() or l[0] == "." or (t.startswith("0x") and l[0] in "ABCDEFabcdef")):
                    if l[0] == ".":
                        if seen_dot:
                            break
                        seen_dot = True
                    t += l[0]
                    l = l[1:]
                if l.startswith("e") or l.startswith("E"):
                    t += l[0]
                    l = l[1:]
                    if l[0] in ("+", "-"):
                        t += l[0]
                        l = l[1:]
                    while l and l[0].isdigit():
                        t += l[0]
                        l = l[1:]
                if l.startswith("j"):
                    t += l[0]
                    l = l[1:]
                yield TokenInfo(NUMBER, t, lineno, 0, org_l)
            elif l[0].isalpha() or l.startswith("_"):
                name = ""
                while l and (l[0].isalpha() or l[0].isdigit() or l.startswith("_")):
                    name += l[0]
                    l = l[1:]
                if (l.startswith('"') or l.startswith("'")) and name in ("b", "r", "rb", "br", "u", "f"):
                    s, l, lineno_delta = get_str(l, readline)
                    yield TokenInfo(STRING, name + s, lineno, 0, org_l)
                    lineno += lineno_delta
                else:
                    yield TokenInfo(NAME, name, lineno, 0, org_l)
            elif l == "\\\n":
                l = readline()
                lineno += 1
            elif l[0] == "\n":
                if paren_level > 0:
                    yield TokenInfo(NL, "\n", lineno, 0, org_l)
                else:
                    yield TokenInfo(NEWLINE, "\n", lineno, 0, org_l)
                break
            elif l[0].isspace():
                l = l[1:]
            elif l.startswith('"') or l.startswith("'"):
                s, l, lineno_delta = get_str(l, readline)
                yield TokenInfo(STRING, s, lineno, 0, org_l)
                lineno += lineno_delta
            elif l.startswith("#"):
                yield TokenInfo(COMMENT, l.rstrip("\n"), lineno, 0, org_l)
                l = "\n"
            else:
                for op in (
                    "**=",
                    "//=",
                    ">>=",
                    "<<=",
                    "+=",
                    "-=",
                    "*=",
                    "/=",
                    "%=",
                    "@=",
                    "&=",
                    "|=",
                    "^=",
                    "**",
                    "//",
                    "<<",
                    ">>",
                    "==",
                    "!=",
                    ">=",
                    "<=",
                    "...",
                    "->",
                ):
                    if l.startswith(op):
                        yield TokenInfo(OP, op, lineno, 0, org_l)
                        l = l[len(op) :]
                        break
                else:
                    yield TokenInfo(OP, l[0], lineno, 0, org_l)
                    if l[0] in ("(", "[", "{"):
                        paren_level += 1
                    elif l[0] in (")", "]", "}"):
                        paren_level -= 1
                    l = l[1:]

    while indent_stack[-1] > 0:
        yield TokenInfo(DEDENT, "", lineno, 0, "")
        indent_stack.pop()

    yield TokenInfo(ENDMARKER, "", lineno, 0, "")



"""
Module: 'math' on micropython-v1.18-esp8266
"""
# MCU: {'ver': 'v1.18', 'port': 'esp8266', 'arch': 'xtensa', 'sysname': 'esp8266', 'release': '1.18', 'name': 'micropython', 'mpy': 9733, 'version': '1.18', 'machine': 'ESP module with ESP8266', 'build': '', 'nodename': 'esp8266', 'platform': 'esp8266', 'family': 'micropython'}
# Stubber: 1.5.4
from typing import Any


def pow(*args, **kwargs) -> Any:
    ...


def acos(*args, **kwargs) -> Any:
    ...


def asin(*args, **kwargs) -> Any:
    ...


def atan(*args, **kwargs) -> Any:
    ...


def atan2(*args, **kwargs) -> Any:
    ...


def ceil(*args, **kwargs) -> Any:
    ...


def copysign(*args, **kwargs) -> Any:
    ...


def cos(*args, **kwargs) -> Any:
    ...


def degrees(*args, **kwargs) -> Any:
    ...


e = 2.71828  # type: float


def exp(*args, **kwargs) -> Any:
    ...


def fabs(*args, **kwargs) -> Any:
    ...


def floor(*args, **kwargs) -> Any:
    ...


def fmod(*args, **kwargs) -> Any:
    ...


def frexp(*args, **kwargs) -> Any:
    ...


def isfinite(*args, **kwargs) -> Any:
    ...


def isinf(*args, **kwargs) -> Any:
    ...


def isnan(*args, **kwargs) -> Any:
    ...


def ldexp(*args, **kwargs) -> Any:
    ...


def log(*args, **kwargs) -> Any:
    ...


def modf(*args, **kwargs) -> Any:
    ...


pi = 3.14159  # type: float


def radians(*args, **kwargs) -> Any:
    ...


def sin(*args, **kwargs) -> Any:
    ...


def sqrt(*args, **kwargs) -> Any:
    ...


def tan(*args, **kwargs) -> Any:
    ...


def trunc(*args, **kwargs) -> Any:
    ...



from typing import Any

class FlashBdev:
    SEC_SIZE: int
    start_sec: Any
    blocks: Any
    def __init__(self, start_sec, blocks) -> None: ...
    def readblocks(self, n, buf, off: int = ...) -> None: ...
    def writeblocks(self, n, buf, off: Any | None = ...) -> None: ...
    def ioctl(self, op, arg): ...

size: Any
bdev: Any
start_sec: Any



"""
Module: 'usys' on micropython-v1.18-esp8266
"""
# MCU: {'ver': 'v1.18', 'port': 'esp8266', 'arch': 'xtensa', 'sysname': 'esp8266', 'release': '1.18', 'name': 'micropython', 'mpy': 9733, 'version': '1.18', 'machine': 'ESP module with ESP8266', 'build': '', 'nodename': 'esp8266', 'platform': 'esp8266', 'family': 'micropython'}
# Stubber: 1.5.4
from typing import Any

argv = []  # type: list
byteorder = "little"  # type: str


def exit(*args, **kwargs) -> Any:
    ...


implementation = ()  # type: tuple
maxsize = 2147483647  # type: int
modules = {}  # type: dict
path = []  # type: list
platform = "esp8266"  # type: str


def print_exception(*args, **kwargs) -> Any:
    ...


stderr: Any  ## <class 'FileIO'> = <io.FileIO 2>
stdin: Any  ## <class 'FileIO'> = <io.FileIO 0>
stdout: Any  ## <class 'FileIO'> = <io.FileIO 1>
version = "3.4.0"  # type: str
version_info = ()  # type: tuple
